<?php
/**
 * Array renderer for PHP_CompatInfo component.
 *
 * PHP versions 4 and 5
 *
 * LICENSE: This source file is subject to version 3.01 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_01.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category PHP
 * @package  PHP_CompatInfo
 * @author   Laurent Laville <pear@laurent-laville.org>
 * @license  http://www.php.net/license/3_01.txt  PHP License 3.01
 * @version  CVS: $Id: Xml.php,v 1.2 2008/06/03 19:36:44 farell Exp $
 * @link     http://pear.php.net/package/PHP_CompatInfo
 * @since    File available since Release 1.8.0b2
 */

require_once 'XML/Util.php';

/**
 * The PHP_CompatInfo_Renderer_Xml class is a concrete implementation
 * of PHP_CompatInfo_Renderer abstract class. It simply display results as
 * an XML stream.
 *
 * @category PHP
 * @package  PHP_CompatInfo
 * @author   Laurent Laville <pear@laurent-laville.org>
 * @license  http://www.php.net/license/3_01.txt  PHP License 3.01
 * @version  Release: @package_version@
 * @link     http://pear.php.net/package/PHP_CompatInfo
 * @since    Class available since Release 1.8.0b2
 */
class PHP_CompatInfo_Renderer_Xml extends PHP_CompatInfo_Renderer
{
    /**
     * Data source parsed final results
     *
     * @var    array
     * @access public
     */
    var $parseData;

    /**
     * A hash containing any additional configuration
     * (Options to improve render with help of XML_Beutifier)
     *
     * @var    array
     * @access public
     */
    var $conf;

    /**
     * Xml Renderer Class constructor (ZE1) for PHP4
     *
     * @param object &$parser Instance of the parser (model of MVC pattern)
     * @param array  $conf    A hash containing any additional configuration
     *
     * @access public
     * @since  version 1.8.0b2 (2008-06-03)
     */
    function PHP_CompatInfo_Renderer_Xml(&$parser, $conf)
    {
        $this->__construct($parser, $conf);
    }

    /**
     * Xml Renderer Class constructor (ZE2) for PHP5+
     *
     * @param object &$parser Instance of the parser (model of MVC pattern)
     * @param array  $conf    A hash containing any additional configuration
     *
     * @access public
     * @since  version 1.8.0b2 (2008-06-03)
     */
    function __construct(&$parser, $conf)
    {
        parent::PHP_CompatInfo_Renderer($parser, $conf);
        $this->conf = $conf;
    }

    /**
     * Updates view with current informations
     *
     * @param object &$auditEvent Instance of Event_Dispatcher
     *
     * @access public
     * @return void
     * @since  version 1.8.0b2 (2008-06-03)
     */
    function update(&$auditEvent)
    {
        $notifyName = $auditEvent->getNotificationName();
        $notifyInfo = $auditEvent->getNotificationInfo();

        switch ($notifyName) {
        case PHP_COMPATINFO_EVENT_AUDITSTARTED :
            parent::startWaitProgress($notifyInfo['dataCount']);
            break;
        case PHP_COMPATINFO_EVENT_AUDITFINISHED :
            parent::endWaitProgress();
            $this->display();
            break;
        case PHP_COMPATINFO_EVENT_FILESTARTED :
            parent::stillWaitProgress($notifyInfo['filename'],
                                      $notifyInfo['fileindex']);
            break;
        case PHP_COMPATINFO_EVENT_FILEFINISHED :
        case PHP_COMPATINFO_EVENT_CODEFINISHED :
            $this->parseData = $notifyInfo;
            break;
        }
    }

    /**
     * Display final results
     *
     * Display final results, when data source parsing is over.
     *
     * @access public
     * @return void
     * @since  version 1.8.0b2 (2008-06-03)
     */
    function display()
    {
        $version    = isset($this->conf['xml']['version'])
                    ? $this->conf['xml']['version'] : '1.0';
        $encoding   = isset($this->conf['xml']['encoding'])
                    ? $this->conf['xml']['encoding'] : 'UTF-8';
        $standalone = isset($this->conf['xml']['standalone'])
                    ? $this->conf['xml']['standalone'] : null;

        $msg  = XML_Util::getXMLDeclaration($version, $encoding, $standalone);
        $msg .= PHP_EOL;
        $msg .= XML_Util::createStartElement('pci',
                                       array('version' => '@package_version@'));

        $o = 15; //$this->_output_level;

        $dataSource = $this->_parser->dataSource['dataSource'];
        $dataType   = $this->_parser->dataSource['dataType'];

        if ($dataType == 'directory') {
            // parsing a directory

            // print <dir> tag
            $tag = array('qname' => 'dir',
                         'content' => $dataSource);
            $msg .= XML_Util::createTagFromArray($tag);
            $msg .= PHP_EOL;

            // print global <version> tag
            if (empty($this->parseData['max_version'])) {
                $attr = array();
            } else {
                $attr = array('max' => $this->parseData['max_version']);
            }
            $tag = array('qname' => 'version',
                         'attributes' => $attr,
                         'content' => $this->parseData['version']);
            $msg .= XML_Util::createTagFromArray($tag);
            $msg .= PHP_EOL;

            // print global <conditions> tag group
            if ($o & 1) {
                $msg .= $this->_printTagList($this->parseData['cond_code'], 'condition');
            }
            // print global <extensions> tag group
            if ($o & 2) {
                $msg .= $this->_printTagList($this->parseData['extensions'], 'extension');
            }
            // print global <constants> tag group
            if ($o & 4) {
                $msg .= $this->_printTagList($this->parseData['constants'], 'constant');
            }
            // print global <tokens> tag group
            if ($o & 8) {
                $msg .= $this->_printTagList($this->parseData['tokens'], 'token');
            }

            // print global <ignored> tag group
            $msg .= XML_Util::createStartElement('ignored');
            $msg .= PHP_EOL;
            // with children groups <files>, <functions>, <extensions>, <constants>
            $ignored = array('file' => $this->parseData['ignored_files'],
                             'function' => $this->parseData['ignored_functions'],
                             'extension' => $this->parseData['ignored_extensions'],
                             'constant' => $this->parseData['ignored_constants']);
            foreach ($ignored as $tag => $data) {
                $msg .= $this->_printTagList($data, $tag);
            }
            $msg .= XML_Util::createEndElement('ignored');
            $msg .= PHP_EOL;

            // remove summary data
            unset($this->parseData['ignored_files']);
            unset($this->parseData['ignored_functions']);
            unset($this->parseData['ignored_extensions']);
            unset($this->parseData['ignored_constants']);
            unset($this->parseData['max_version']);
            unset($this->parseData['version']);
            unset($this->parseData['functions']);
            unset($this->parseData['extensions']);
            unset($this->parseData['constants']);
            unset($this->parseData['tokens']);
            unset($this->parseData['cond_code']);

            $files = $this->parseData;
        } elseif ($dataType == 'file') {
            // parsing a single file
            $files = array($dataSource => $this->parseData);
        } else {
            // ... or a chunk of code (string)
            $files = array($this->parseData);
        }

        if ($dataType != 'string') {
            // print <files> tag group
            $msg .= XML_Util::createStartElement('files',
                                                 array('count' => count($files)));
            $msg .= PHP_EOL;
        }

        foreach ($files as $file => $this->parseData) {
            if ($dataType == 'string') {
                $msg .= XML_Util::createStartElement('string');
            } else {
                // print local <file> tag
                $msg .= XML_Util::createStartElement('file', array('name' => $file));
            }
            $msg .= PHP_EOL;

            // print local <version> tag
            if (empty($this->parseData['max_version'])) {
                $attr = array();
            } else {
                $attr = array('max' => $this->parseData['max_version']);
            }
            $tag = array('qname' => 'version',
                         'attributes' => $attr,
                         'content' => $this->parseData['version']);
            $msg .= XML_Util::createTagFromArray($tag);
            $msg .= PHP_EOL;

            // print local <conditions> tag group
            if ($o & 1) {
                $msg .= $this->_printTagList($this->parseData['cond_code'], 'condition');
            }
            // print local <extensions> tag group
            if ($o & 2) {
                $msg .= $this->_printTagList($this->parseData['extensions'], 'extension');
            }
            // print local <constants> tag group
            if ($o & 4) {
                $msg .= $this->_printTagList($this->parseData['constants'], 'constant');
            }
            // print local <tokens> tag group
            if ($o & 8) {
                $msg .= $this->_printTagList($this->parseData['tokens'], 'token');
            }

            // print local <ignored> tag group
            $msg .= XML_Util::createStartElement('ignored');
            $msg .= PHP_EOL;
            // with children groups <functions>, <extensions>, <constants>
            $ignored = array('function' => $this->parseData['ignored_functions'],
                             'extension' => $this->parseData['ignored_extensions'],
                             'constant' => $this->parseData['ignored_constants']);
            foreach ($ignored as $tag => $data) {
                $msg .= $this->_printTagList($data, $tag);
            }
            $msg .= XML_Util::createEndElement('ignored');
            $msg .= PHP_EOL;

            // verbose level
            $v = 1; //$this->args->getValue('v');

            // extra information only if verbose mode >= 4
            if ($v & 4) {
                unset($this->parseData['ignored_files']);
                unset($this->parseData['ignored_functions']);
                unset($this->parseData['ignored_extensions']);
                unset($this->parseData['ignored_constants']);
                unset($this->parseData['max_version']);
                unset($this->parseData['version']);
                unset($this->parseData['functions']);
                unset($this->parseData['extensions']);
                unset($this->parseData['constants']);
                unset($this->parseData['tokens']);
                unset($this->parseData['cond_code']);

                // print local <functions> tag group
                $msg .= $this->_printTagList($this->parseData, 'function');
            }

            if ($dataType == 'string') {
                $msg .= XML_Util::createEndElement('string');
            } else {
                $msg .= XML_Util::createEndElement('file');
            }
            $msg .= PHP_EOL;
        }
        if ($dataType != 'string') {
            $msg .= XML_Util::createEndElement('files');
            $msg .= PHP_EOL;
        }
        $msg .= XML_Util::createEndElement('pci');
        $msg .= PHP_EOL;

        // try to see if we can improve XML render
        $beautifier = 'XML/Beautifier.php';
        if (PHP_CompatInfo_Renderer::isIncludable($beautifier)) {
            include_once $beautifier;
            $def = array();
            $opt = isset($this->conf['beautifier'])
                 ? $this->conf['beautifier'] : $def;
            $fmt = new XML_Beautifier($opt);
            $msg = $fmt->formatString($msg);
        }

        echo $msg;
    }

    /**
     * Print a group of same tag in the XML report.
     *
     * Groups list are : extension(s), constant(s), token(s)
     *
     * @param array  $dataSrc Data source
     * @param string $tagName Name of the XML tag
     *
     * @return string
     * @access private
     * @since  version 1.7.0b4 (2008-04-03)
     */
    function _printTagList($dataSrc, $tagName)
    {
        $msg = '';

        if ($tagName == 'function') {
            $c = 0;
            foreach ($dataSrc as $version => $functions) {
                $c += count($functions);
            }
            $attributes = array('count' => $c);
        } elseif ($tagName == 'condition') {
            if ($this->_parser->options['debug'] === true) {
                $c = 0;
                foreach ($dataSrc[1] as $cond => $elements) {
                    $c += count($elements);
                }
                $attributes = array('count' => $c, 'level' => $dataSrc[0]);
            } else {
                $attributes = array('level' => $dataSrc[0]);
            }
        } else {
            $attributes = array('count' => count($dataSrc));
        }

        $msg .= XML_Util::createStartElement($tagName.'s', $attributes);
        $msg .= PHP_EOL;

        if ($tagName == 'function') {
            foreach ($dataSrc as $version => $functions) {
                foreach ($functions as $data) {
                    $attr = array('version' => $version);
                    if (!empty($data['extension'])) {
                        $attr['extension'] = $data['extension'];
                        $attr['pecl']      = $data['pecl'] === true ?
                                                'true' : 'false';
                    }
                    $tag = array('qname' => $tagName,
                                 'attributes' => $attr,
                                 'content' => $data['function']);
                    $msg .= XML_Util::createTagFromArray($tag);
                    $msg .= PHP_EOL;
                }
            }
        } elseif ($tagName == 'condition') {
            if ($this->_parser->options['debug'] == true) {
                foreach ($dataSrc[1] as $cond => $elements) {
                    $cond = ($cond == 0) ? 1 : ($cond * 2);
                    foreach ($elements as $data) {
                        $tag = array('qname' => $tagName,
                                     'attributes' => array('level' => $cond),
                                     'content' => $data);
                        $msg .= XML_Util::createTagFromArray($tag);
                        $msg .= PHP_EOL;
                    }
                }
            }
        } else {
            foreach ($dataSrc as $data) {
                $tag = array('qname' => $tagName,
                             'attributes' => array(),
                             'content' => $data);
                $msg .= XML_Util::createTagFromArray($tag);
                $msg .= PHP_EOL;
            }
        }

        $msg .= XML_Util::createEndElement($tagName.'s');
        $msg .= PHP_EOL;

        return $msg;
    }
}
?>